from visual import *
#from Numeric import *

print '''
Simple solar system model showing the orbits of the planets
Note: In this version the orbits are not tilted so
      the longitude of the ascending node is not correct.
created by Lensyl Urbano, Jan. 2006.
'''



class orbit:

    def __init__(self,e=0.0,rad=1.0, mu=10.0, planet_size=1.0, period=365,
                 inclination=0.0, lap=0.0, frate=10000000):
        self.orbit_path = curve(color=color.green,radius=0.075)
        self.planet = sphere(color=color.yellow,radius=planet_size)
        if e >= 1.0:
            e = 0.99
        if e < 0.0:
            e = 0.0
        fx = 2.0 - 2.0*min(abs((1-e*e)/(1-e)), abs(-(1-e*e)/(1+e)))
        self.fx = fx
        self.e = e
        self.rad = rad
        self.dangle = 360./period
        self.inclination = inclination
        self.lap = lap
        for i in range(int(period)+1):
            rate(frate)
            theta = radians(float(360.*i/period))
            #h_sq = a*(1-e*e)*mu
            r = (1-e*e)/(1-e*cos(theta))
            nx = rad*(r*cos(theta) - fx)
            ny = rad*r*sin(theta) 

            #print i, f, r, cos(theta), sin(theta), nx, ny
            #nz = rotate(vector(nx,ny,0.0), angle=inclination, 
            posi = rotate(vector(nx,ny,0.0), angle=radians(inclination), axis=(0,1,0))
            posi = rotate(posi, angle=radians(lap), axis=(0,0,1))
            
            self.orbit_path.append(pos=posi)
            self.planet.pos=posi
        self.angle = 0.0

    def re_draw(self,e=0.0,rad=1.0,mu=10.0, frate=10000000):
        #redraw last path
        if e >= 1.0:
            e = 0.99
        if e < 0.0:
            e = 0.0
        fx = 2.0 - 2.0*min(abs((1-e*e)/(1-e)), abs(-(1-e*e)/(1+e)))
        self.fx = fx
        self.e = e
        self.rad = rad
        for i in range(361):
            rate(frate)
            theta = radians(float(i))
            #h_sq = a*(1-e*e)*mu
            r = (1-e*e)/(1-e*cos(theta))
            nx = rad*(r*cos(theta) - fx)
            ny = rad*r*sin(theta) 

            
            self.orbit_path.pos[i]=vector(nx, ny, 0.0)
            


    def move_planet(self):
        self.angle = self.angle + self.dangle
        #to show the planet orbiting the Sun
        theta = radians(float(self.angle))
        r = (1-self.e*self.e)/(1-self.e*cos(theta))
        nx = self.rad*(r*cos(theta) - self.fx)
        ny = self.rad*r*sin(theta) 
        posi=rotate(vector(nx,ny,0.0), angle=radians(self.inclination), axis=(0,1,0))
        posi = rotate(posi, angle=radians(self.lap), axis=(0,0,1))
        self.planet.pos = posi


#class proton:
    

origx = 0
origy = 0
w = 704 #+4+4
h = 576 #+24+4
scene.width=w
scene.height=h
scene.x = origx
scene.y = origy

mu = 10.0 #gravitational acceleration
#e = 0.6 #eccentricity
#a = 10.0 #distance of sun from center of elipse

# to draw an elipse
prad = 1.0

nucleus = frame()
p1 = sphere(color=color.red, frame=nucleus, radius=prad, pos=(-prad,0,0))
p2 = sphere(color=color.red, frame=nucleus, radius=prad, pos=(prad,0,0))
p3 = sphere(color=color.red, frame=nucleus, radius=prad, pos=(0,prad,0))
n1 = sphere(color=color.blue, frame=nucleus, radius=prad, pos=(0,0,prad))
n2 = sphere(color=color.blue, frame=nucleus, radius=prad, pos=(0,0,-prad))
n2 = sphere(color=color.blue, frame=nucleus, radius=prad, pos=(0,-prad,0))
#orbitor = sphere()

e1 = orbit(e=0.0, rad=15.0, mu=mu, 
              planet_size=0.5, period=90,
              lap=90)
e2 = orbit(e=0.0, rad=15.0, mu=mu, 
              planet_size=1.0, period=90,
              lap=0, inclination=90)

e3 = orbit(e=0.0, rad=30.0, mu=mu, 
              planet_size=1.0, period=90,
              lap=0, inclination=0)

runtime = 0.0
framerate = 20
pick = None

scene.autoscale = 0

while 1:
    rate(framerate)
    runtime += 1.0/framerate

    e1.move_planet()
    e2.move_planet()
    e3.move_planet()

    scene.forward = rotate(scene.forward, angle=1*pi/180.0, axis=(0,1,0))
